# Copyright 2000 by Brad Chapman.  All rights reserved.
# This code is part of the Biopython distribution and governed by its
# license.  Please see the LICENSE file that should have been included
# as part of this package.

# standard library
import string

# stubs generated from the idl
import BioCorba.org__POA.biocorba.seqcore as seqcore__POA
import BioCorba.org.biocorba.seqcore as seqcore

# import the class to inheret from
from CorbaAnonymousSeq import CorbaAnonymousSeq

class CorbaPrimarySeq(seqcore__POA.PrimarySeq, CorbaAnonymousSeq):
    """Represent a sequence and its ids and versions.

    This class inherets from the following classes:
    * org__POA.biocorba.seqcore.PrimarySeq - The corba stub code
    implementing code for connecting through corba (this is
    generated by the orb implementation.)
    * CorbaAnonymousSeq - The idl defined class
    from which this class inherets. AnonymousSeq provides functions
    for dealing with the sequence directly, while this class provides
    id information.
    """
    def __init__(self, seq_info):
        """Initialize a PrimarySeq object.

        Arguments:
        * seq_info - A SeqRecord object which contains the sequence
        to represent, along with ids and descriptions.
        """
        # initialize AnonymousSeq with the basic sequence info
        self._seq_record = seq_info
        CorbaAnonymousSeq.__init__(self, seq_info.seq)

    def unref(self):
        """Use unref to get rid of the reference to our SeqRecord object.
        """
        self._seq_record = None
        CorbaAnonymousSeq.unref(self)

    def display_id(self):
        """Return the human-readible id for the sequence.
        """
        return self._seq_record.description

    def primary_id(self):
        """Return the unique id for the sequence.
        """
        return self._seq_record.name

    def accession_number(self):
        """Return the unique accession number id for the sequence.
        """
        return self._seq_record.id

    def version(self):
        """Return the version number for the sequence.

        If we have an accession number that has a period (ie.
        AC00001.2) we return the 2 as the version number, otherwise
        return 0 to signify the most recent version.
        """
        if self._seq_record.id is not None:
            if string.find(self._seq_record.id, '.') != -1:
                id, version = string.split(self._seq_record.id, '.')
                return int(version) 
            # default, just return 0
            else:
                return 0
        else:
            return 0
