"""Represent databases holding biological information.

Databases are the most advanced BioCorba object and are analagous to
python dictionaries in that they can be accessed via keys, which are
accession numbers of the sequences.

classes:
o CorbaPrimarySeqDB
o CorbaSeqDB
o CorbaUpdateableSeqDB
"""
# stubs generated from the idl
import BioCorba.org__POA.biocorba.seqcore as seqcore__POA
import BioCorba.org.biocorba.seqcore as seqcore

# the class we inheret from
from BioCorba.Server.GnomeMemory.CorbaUnknown import CorbaUnknown

# classes we will be returning instances of
from Vector import CorbaPrimarySeqVector
from CorbaPrimarySeq import CorbaPrimarySeq
from CorbaSeq import CorbaSeq

# configuration variables
from BioCorba.biocorbaconfig import *

class CorbaPrimarySeqDB(seqcore__POA.PrimarySeqDB, CorbaUnknown):
    """PrimarySeqDB BioCorba implementation.

    This class inherits from:
    * org__POA.biocorba.seqcore.PrimarySeqDB - The corba stub code
    implementing code for connecting through corba (this is
    generated by the orb implementation.)
    * CorbaUnknown - The memory management class from which this
    inherits in the idl.
    """
    def __init__(self, pseq_dict, db_name = 'Generic Database',
                 db_version = 0):
        """Initialize a database to return PrimarySeq objects.

        Arguments:
        o pseq_dict - A dictionary-like object that returns SeqRecord
        objects. The object must implement __getitem__ (by name) and
        keys().
        o db_name - The name of the database.
        o db_version - The database version (must be an integer).
        """
        CorbaUnknown.__init__(self)
        self._object_dict = pseq_dict
        self._name = db_name
        self._version = int(db_version)

    def unref(self):
        """Use unref to get rid of a reference to our loaded dictionary.
        """
        self._object_dict = None
        CorbaUnknown.unref(self)

    def name(self):
        return self._name

    def version(self):
        return self._version

    def max_sequence_length(self):
        """Return the maximum length of a sequence that can be returned.

        This size is defined in the biocorba configuration file.
        """
        return MAX_SEQ_SIZE

    def get_PrimarySeq(self, acc_num, version = 0):
        """Retrieve a PrimarySeq object by its accession number.
        """
        try:
            seq_record_item = self._object_dict[acc_num]
        except KeyError:
            raise seqcore.DoesNotExist \
                  ("Could not locate accession number %s" % acc_num)

        pseq_item = CorbaPrimarySeq(seq_record_item)

        # check the versioning to make sure we are right
        item_version = pseq_item.version()

        if version != 0 and item_version != 0:
            if version != item_version:
                raise seqcore.DoesNotExist \
                      ("Version mismatch. Wanted %s, Got %s"
                       % (version, item_version))

        return pseq_item.get_object()
        
    def get_PrimarySeqVector(self):
        """Retrieve a vector of all of the PrimarySeq objects in the db.
        """
        all_seq_records = []
        all_keys = self._object_dict.keys()
        all_keys.sort()
        for pseq_key in all_keys:
            all_seq_records.append(self._object_dict[pseq_key])

        pseq_vector = CorbaPrimarySeqVector(all_seq_records)
        return pseq_vector.get_object()

class CorbaSeqDB(seqcore__POA.SeqDB, CorbaPrimarySeqDB):
    """SeqDB BioCorba implementation.

    This class inherets from:
    * org__POA.biocorba.seqcore.SeqDB - The corba stub code
    implementing code for connecting through corba (this is
    generated by the orb implementation.)
    * CorbaPrimarySeqDB - The primary sequence database interface
    from which this class inherits in the idl.
    """
    def __init__(self, seq_dict, db_name = 'Generic Database',
                 db_version = 0.0):
        """Initialize a database to return PrimarySeq objects.

        Arguments:
        o seq_dict - A dictionary-like object that returns SeqRecord
        objects. The object must implement __getitem__ (by name) and
        keys().
        o db_name - The name of the database.
        o db_version - The database version (must be a number).
        """
        CorbaPrimarySeqDB.__init__(self, seq_dict, db_name, db_version)

    def get_Seq(self, acc_num, version = 0):
        try:
            seq_record_item = self._object_dict[acc_num]
        except KeyError:
            raise seqcore.DoesNotExist \
                  ("Could not locate accession number %s." % acc_num)

        seq_item = CorbaSeq(seq_record_item)

        # check the versioning to make sure we have the right version
        item_version = seq_item.version()

        # if either version is zero then we can't really compare and
        # have to assume they are right, otherwise make sure they're the same
        if version != 0 and item_version != 0:
            if version != item_version:
                raise seqcore.DoesNotExist \
                      ("Version mismatch. Wanted %s, Got %s"
                       % (version, item_version))

        return seq_item.get_object()

    def accession_numbers(self):
        """Retrieve all of the accession numbers (keys) of the database.
        """
        all_keys = self._object_dict.keys()
        all_keys.sort()
        return all_keys

class CorbaUpdateableSeqDB(CorbaSeqDB):
    """A mutable sequence database.
    """
    def __init__(self, seq_dict, db_name = 'Generic Database',
                 db_version = 0.0):
        """Initialize a database to return PrimarySeq objects.

        Arguments:
        o seq_dict - A dictionary-like object that returns SeqRecord
        objects. The object must implement __getitem__ (by name) and
        keys().
        o db_name - The name of the database.
        o db_version - The database version (must be a number).
        """
        CorbaSeqDB.__init__(seq_dict, db_name, db_version)

    def write_Seq(self, updated_seqs, added_seqs, dead_seqs):
        print "UpdateableSeqDB::write_Seq is not implemented."

    def write_PrimarySeq(self, updated_seqs, added_seqs, dead_seqs):
        print "UpdateableSeqDB::write_PrimarySeq is not implemented."
